"""Django application configuration for carbondesign.
"""
from django.apps import AppConfig


class CarbonDesignConfig(AppConfig):
    """Django application configuration for carbondesign.
    """
    name = 'carbondesign'
